@@ -14,11 +14,10 @@ In F<config.yaml>
 
   actions:
     ...
-    - rss
+    rss:
+      class: SVN::Web::RSS
     ...
 
-  rss_class: SVN::Web::RSS
-
 =head1 DESCRIPTION
 
 Generates an RSS feed of commits to a file or path in the Subversion 
